from sapphirerenderer import SapphireRenderer
import numpy as np
from time import sleep


def main():
    renderer = SapphireRenderer(draw_axis=True)

    cubes = []

    # 4800 vertices
    for i in range(600):
        cubes.append(
            renderer.add_object(
                "Cube",
                [
                    np.random.rand(3) * 20,
                    (0, 0, 0),
                    True,
                ],
            )
        )

    for cube in cubes:
        cube.rotate_around_point(45.0, 0.0, 0.0, np.array([0, 0, 0]))
        continue

    while renderer.running:
        for cube in cubes:
            cube.rotate_local(0, 1, 0)
        sleep(0.01)


if __name__ == "__main__":
    main()



from ..object_classes.flat_faces_object import FlatFacesObject
import numpy as np


class Grass(FlatFacesObject):
    def __init__(
        self,
        position=np.array([0.0, 0.0, 0.0]),
        color=(0, 0, 0),
        size=1,
        rows=5,
        cols=5,
        z_randomness=0.1,
        color_randomness=0.1,
        shadow_effect=1,
    ):
        # Generate vertices for a flat grid
        x_vals = np.linspace(0, size, cols)
        y_vals = np.linspace(0, size, rows)
        vertices = np.array([(x, y, 0) for x in x_vals for y in y_vals], dtype=float)

        # Adding randomness to Z height of vertices
        vertices[:, 2] += np.random.uniform(
            -z_randomness, z_randomness, size=vertices.shape[0]
        )

        # Generate faces for the grid
        faces = []
        for i in range(rows - 1):
            for j in range(cols - 1):
                idx = i * cols + j
                v0_idx = idx
                v1_idx = idx + 1
                v2_idx = idx + cols + 1
                v3_idx = idx + cols
                face_indices = [v0_idx, v1_idx, v2_idx, v3_idx]
                face_color = (0, 255, 0)  # Green color

                # Adding slight variations to green color
                color_array = np.array(face_color, dtype=float)
                color_array += np.random.uniform(
                    -color_randomness * 255, color_randomness * 255, size=3
                )
                color_array = np.clip(color_array, 0, 255).astype(int)

                # Calculate face normal
                v0 = vertices[v0_idx]
                v1 = vertices[v1_idx]
                v2 = vertices[v2_idx]
                normal = np.cross(v1 - v0, v2 - v0)
                normal = normal / np.linalg.norm(normal) * 255

                normal = -normal

                faces.append((face_indices, tuple(color_array), normal))

        super().__init__(vertices, faces, position, color, True, shadow_effect)



from sapphirerenderer import SapphireRenderer
import numpy as np
from time import sleep


def main():
    renderer = SapphireRenderer(draw_axis=True)

    cubes = []

    # 2400 vertices
    for i in range(300):
        cubes.append(
            renderer.add_object(
                "Fcube",
                [
                    np.random.rand(3) * 20,
                    (0, 0, 0),
                    True,
                ],
            )
        )

    while renderer.running:
        for cube in cubes:
            cube.rotate_around_point(0.0, 0.1, 0.1, np.array([0, 0, 0]))
        sleep(0.1)


if __name__ == "__main__":
    main()



from ..object_classes.base_object import Object
import numpy as np
import pygame
from ..point_math.project_point import project_point
from ..point_math.average_points import average_points

pygame.init()


class SurfaceObject(Object):
    def __init__(self, vertices, color=(0, 0, 0)):
        """
        Object with a single surface
        :param vertices: the vertices of the surface
        :param color: the color of the surface
        """
        self.position = average_points(vertices)
        super().__init__(color=color, position=self.position)
        self.vertices = vertices

        self.drawing = False
        self.ambiguous = False

        self.show()

    def change_vertices(self, vertices):
        """
        Change the vertices of the surface
        :param vertices: the new vertices
        :return:
        """
        self._wait_for_draw()

        self.ambiguous = True
        self.vertices = vertices
        self.position = average_points(vertices)
        self.ambiguous = False

    def draw(self, surface, camera):
        """
        Draw the surface
        :param surface: the pygame surface to draw on
        :param camera: the camera to draw from
        :return:
        """
        self._wait_for_ambiguous()

        moved_vertices = self.vertices - camera.position
        reshaped_vertices = moved_vertices.reshape(-1, 1, moved_vertices.shape[1])
        rotated_vertices = np.sum(camera.rotation_matrix * reshaped_vertices, axis=-1)

        projected_vertices = [
            project_point(
                vertex,
                camera.offset_array,
                camera.focal_length,
            )[0]
            for vertex in rotated_vertices
        ]

        for vertex in projected_vertices:
            if vertex is None:
                return

        pygame.draw.polygon(surface, self.color, projected_vertices)



from sapphirerenderer import SapphireRenderer
import numpy as np
from time import sleep


def main():
    renderer = SapphireRenderer(draw_axis=True)

    stl = renderer.add_object(
        "Fstl",
        [
            "O:\Python Files\Packages\Sapphire-Renderer\src\sapphirerenderer\objects\suzanne.stl",
            np.array([0, 0, 0]),
            (255, 0, 0),
            False,
        ],
    )

    grass = renderer.add_object(
        "Grass", args=[np.array([0, 0, 0]), (255, 0, 0), 5, 10, 10, 0.1, 0]
    )

    grass.move_absolute(np.array([-2.5, -2.5, 0]))

    stl.move_absolute(np.array([0, 0, 1]))

    while renderer.running:
        # stl.rotate_local(0, 0.01, 0)
        sleep(0.01)


if __name__ == "__main__":
    main()
